
function Clamp(value, min, max)
{
  return Math.max(min, Math.min(value, max));
}

/**
 * This lookup does nothing, null, nada.
 * It's useful for color channels that you don't want to change.
 */
function NullLookup()
{
  var lookup = new Array(256);
  for (var i = 0; i < 256; i++) {
    lookup[i] = i;
  }
  return lookup;
}

/**
 * A linear adjust brightness lookup
 * This lookup adds 'value' to each color value
 * it clamps the color within the range: 0 <-> 255
 * @param value the amount of brightness to add, should be within 0 to 255
 */
function BrightenLookup(value)
{
  var lookup = new Array(256);
  for (var i = 0; i < 256; i++) {
    lookup[i] = Math.floor(Clamp(i + value, 0, 255));
  }
  return lookup;
}

/**
 * A color inverter lookup
 * @param min Usually zero
 * @param max Usually 255
 */
function InvertLookup(min, max)
{
  var lookup = new Array(256);
  for (var i = 0; i < 256; i++) {
    lookup[i] = Math.floor(Clamp(((max - i) + min), 0, 255));
  }
  return lookup;
}

/**
 * If the pixel is greater than value, it should be 255, otherwise it should be zero
 */
function ThresholdLookup(value) {
  var lut = new Array(256);
  for (var i = 0; i < 256; i++) {
    lut[i] = (i > value) ? 255 : 0;
  }
  return lut;
} 

/**
 * A solarize lookup
 * @param value The soloralize value from 0 to 255
 */
function SolarizeLookup(value)
{
  var lookup = new Array(256);
  for (var j = 0; j < 256; ++j) {
    lookup[j] = Clamp(((j > value) ? (255 - j) : j), 0, 255);
  }  
  return lookup;
}

/**
 * @param value The number of unique colors in the channel
 */
function LookupColors(value)
{
  var lookup = new Array(256);
  
  switch (value) {
    case 2:
      for (var j = 0; j < 256; ++j) {
        lookup[j] = Math.floor(j/value) * 255;
      }
    break;

    case 4:
      for (var j = 0; j < 256; ++j) {
        lookup[j] = Math.floor(j/value) * 255/2;
      }
    break;

    case 8:
      for (var j = 0; j < 256; ++j) {
        lookup[j] = Math.floor(j/value) * 255/4;
      }
    break;

    case 16:
      for (var j = 0; j < 256; ++j) {
        lookup[j] = Math.floor(j/value) * 255/8;
      }
    break;

    case 256:
      for (var j = 0; j < 256; ++j) {
        lookup[j] = j;
      }
    break;
  }

  return lookup;
}


/**
 * A non-linear adjust brightness lookup
 * This lookup adds
 * it clamps the color within the range: 0 <-> 255
 * @param gamma the amount of gamma to add, value should be within 0.1 to 2.2 (anything greater than 10 is silly)
 */
function GammaLookup(gamma)
{
  var lookup = new Array(256);
  
  if (gamma) {
    var value = 1.0 / gamma;

    for (var i = 0; i < 256; ++i) {
      lookup[i] = Math.floor(Clamp(255.0 * (Math.pow(i / 255.0, value)), 0, 255));
    }  
  }
  else {
    for (var i = 0; i < 256; ++i) {
      lookup[i] = 0;
    }
  }

  return lookup;
}
